package com.google.android.RealMaze;

/*
 * 
 * Original DemoKit application supplied with Google APIs API Level 10 add-on library
 * Source: http://developer.android.com/guide/topics/usb/adk.html#running-demokit
 * 
 * Modified by Paul Hutchison, PC Dev Limited, June 2011 
 * http://www.pcdev.co.nz/
 * 
 *  Thanks to the many developers out there providing code samples - I managed to work 
 *  the accelerometer SensorEventListener into this class without too much trouble
 *  using a number of examples, the most helpful were:
 *  http://mobilestrategist.blogspot.com/2010/01/android-accelerometer-and-orientation.html
 *  and
 *  http://www.anddev.org/code-snippets-for-android-f33/convert-android-accelerometer-values-and-get-tilt-from-accel-t6595.html
 *  
 *  Thanks!
 */

import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;

public class BaseActivity extends RealMazeActivity implements SensorEventListener {
	 
	private SensorManager sensorManager = null;

	private InputController mInputController;
	private double tiltX;
    private double tiltY;
    private double totAcc;
    private double accX;
    private double accY;
    private double accZ;
    
	public BaseActivity() {
		super();
	}

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		if (mAccessory != null) {
			showControls();
	        // Get a reference to a SensorManager
	        sensorManager = (SensorManager) getSystemService(SENSOR_SERVICE);
		} else {
			hideControls();
		}
	}

    @Override
    public void onResume() {
     super.onResume();
     if (sensorManager == null)
    	 sensorManager = (SensorManager) getSystemService(SENSOR_SERVICE);
     // Register this class as a listener for the accelerometer sensor
     sensorManager.registerListener(this, sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER), SensorManager.SENSOR_DELAY_GAME);
    }
 
    @Override
    protected void onStop() {
     // Unregister the listener
     sensorManager.unregisterListener(this);
     super.onStop();
    } 

    public void onDestroy() {
        super.onDestroy();
    }

    public void onAccuracyChanged(Sensor s, int i) {
    	// method must be implemented but we don't need it
    }
    	
    // This method will update the UI on new sensor events
    // thanks to "noya" on anddev.org for the tilt code:
    // http://www.anddev.org/code-snippets-for-android-f33/convert-android-accelerometer-values-and-get-tilt-from-accel-t6595.html
    public void onSensorChanged(SensorEvent sensorEvent) {
     int x,y;
     synchronized (this) {
      if (sensorEvent.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
    	  accX = -(sensorEvent.values[0])/SensorManager.GRAVITY_EARTH;
          accY = -(sensorEvent.values[1])/SensorManager.GRAVITY_EARTH;
          accZ = sensorEvent.values[2]/SensorManager.GRAVITY_EARTH;
          totAcc = Math.sqrt((accX*accX)+(accY*accY)+(accZ*accZ));
    	  tiltX = Math.asin(accX/totAcc);
          tiltY = Math.asin(accY/totAcc);
          // we ignore tilt in the z axis
          
          // convert the x and y tilt angles to something on a roughly usable linear scale (-128 through 128)
          x = (int)(tiltX*1000/(2*Math.PI)); 
          y = (int)(-tiltY*1000/(2*Math.PI));
          
          //update visual display
          updateJoyView(x, y);
          
          // Send absolute position to servo drivers on Arduino board:
          // Note that for some reason the x axis required a multiplier as the servo
          // swing was not long enough.  You can adjust multipliers to suit your setup
          // and hardware.  If you find the servo is moving in reverse to what it 
          // should be then multiply the value by -1.
          sendServoCommand(1, (int)(x*1.5)); 
          sendServoCommand(2, (int)(y*1.0));
		
      }
     }
    }
   
    private void sendServoCommand(int servoNumber, int pos){

    	// translate the (roughly) -128 to 128 range to the range 0-255
    	pos += 128;
    	if(pos < 0)
    		pos = 0;
    	if(pos > 255)
    		pos = 255;
    	
    	// My servos didn't seem to have a full 0-255 sweep,
    	// they start buzzing at about 225.  If you have the 
    	// same problem then uncomment the following line:
    	
    	// if(pos > 225) pos = 225;

		this.sendCommand(RealMazeActivity.LED_SERVO_COMMAND,
				(byte) (servoNumber - 1 + 0x10), (byte) pos);

    }

    @Override
	public boolean onCreateOptionsMenu(Menu menu) {
		menu.add("Quit");
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		if (item.getTitle() == "Quit") {
			finish();
			System.exit(0);
		}
		return true;
	}

	protected void enableControls(boolean enable) {
		if (enable) {
			showControls();
		} else {
			hideControls();
		}
	}

	protected void hideControls() {
		setContentView(R.layout.no_device);
		mInputController = null;
	}

	protected void showControls() {
		setContentView(R.layout.main);

		mInputController = new InputController(this);
		mInputController.accessoryAttached();
	}

	protected void updateJoyView(int x, int y) {
		if (mInputController != null) {
			mInputController.joystickMoved(x, y);
		}
	}
}